@@ -1,4 +1,4 @@
-# Copyright (C) 2006-2007 Warp Networks S.L.
+# Copyright (C) 2008-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -24,11 +24,13 @@ use warnings;
 # Perl6::Junction package
 # Date::Calc::Object  (which has as a dependency Carp::Clan and Bit::Vector)
 # File::Copy::Recursive
+# File::Temp
 ####################################
 use File::Slurp;
 use Perl6::Junction qw(any);
 use Date::Calc::Object qw(:all);
 use File::Copy::Recursive qw(dircopy fcopy);
+use File::Temp;
 
 use base qw(EBox::GConfModule EBox::Model::ModelProvider EBox::Module::Service);
 
@@ -44,8 +46,10 @@ use EBox::Exceptions::DataMissing;
 use EBox::Exceptions::DataInUse;
 use EBox;
 use EBox::CA::Certificates;
+use EBox::Validate;
+use EBox::Sudo;
 
-use constant TEMPDIR     => EBox::Config->tmp(); # "/tmp/";
+use constant TEMPDIR     => "/tmp"; # EBox::Config->tmp();
 use constant OPENSSLPATH => "/usr/bin/openssl";
 
 use constant CATOPDIR => EBox::Config->home() . "CA/";
@@ -61,6 +65,8 @@ use constant NEWCERTSDIR => CATOPDIR . "newcerts/";
 use constant CERTSDIR    => CATOPDIR . "certs/";
 # Place to put the public keys
 use constant KEYSDIR      => CATOPDIR . "keys/";
+# Place to put the P12 keystore
+use constant P12DIR       => CATOPDIR . "p12/";
 use constant CAREQ        => REQDIR   . "careq.pem";
 use constant CACERT       => CATOPDIR . "cacert.pem";
 use constant INDEXFILE    => CATOPDIR . "index.txt";
@@ -73,8 +79,9 @@ use constant CAPRIVKEY   => PRIVDIR . "cakey.pem";
 use constant CAPUBKEY    => KEYSDIR . "capubkey.pem";
 
 # Directory mode to allow only owner and readable from others
-use constant DIRMODE     => 00705;
-use constant PRIVATEDIRMODE     => 00700;
+# Others means openvpn user (nobody)
+use constant DIRMODE        => 00751;
+use constant PRIVATEDIRMODE => 00700;
 
 # Use Certification Version 3
 use constant EXTENSIONS_V3 => "1";
@@ -263,6 +270,7 @@ sub createCA {
     mkdir (KEYSDIR, DIRMODE);
     mkdir (PRIVDIR, PRIVATEDIRMODE);
     mkdir (REQDIR, DIRMODE);
+    mkdir (P12DIR, PRIVATEDIRMODE);
     # Create index and crl number
     open ( my $OUT, ">" . INDEXFILE);
     close ($OUT);
@@ -724,6 +732,16 @@ sub CAPublicKey {
 #                    (Optional)
 #       certFile - path to store the new certificate file (Optional)
 #
+#       subjAltNames - Array ref containing the subject alternative
+#                      names in a hash ref with the following keys:
+#
+#                      type  - String the type must be one of these: DNS, IP or email
+#
+#                      value - String the value following these
+#                              constraints:
+#                         DNS   - Domain name
+#                         IP    - IPv4 address
+#                         email - email address
 #
 # Returns:
 #
@@ -814,6 +832,10 @@ sub issueCertificate
   $dn->attribute("commonName", $args{commonName})
     if (defined($args{commonName}));
 
+  if (defined($args{subjAltNames})) {
+      $self->_checkSubjAltNames($args{subjAltNames});
+  }
+
   # Create the certificate request
   my $genKey = 1;
   if ( defined($args{privateKeyFile} )) {
@@ -845,6 +867,7 @@ sub issueCertificate
                                  userCertFile => $certFile,
 				 selfsigned   => $selfSigned,
 				 endDate      => $userExpDay,
+                                 subjAltNames => $args{subjAltNames},
 			       );
 
   # Check if something goes wrong
@@ -860,6 +883,11 @@ sub issueCertificate
 				   $args{keyPassword});
   }
 
+  # Generate the P12 key store
+  unless ( $selfSigned ) {
+      $self->_generateP12Store($privKey, $self->getCertificateMetadata(cn => $args{commonName}), $args{keyPassword});
+  }
+
   # Logging the action
 #  logAdminNow($self->name, "issueCertificate",
 #		"cn=" . $args{commonName} . ",days=" . $args{days}
@@ -1048,6 +1076,10 @@ sub generateCRL
 #       - isCACert   - boolean indicating that it is the valid CA certificate
 #       - path       - certificate path
 #       - serialNumber - serial number within CA
+#       - subjAltNames - array ref the subject alternative names containing hash ref
+#                         with the following valid types:
+#                        type  => 'DNS', 'IP' and 'email'
+#                        value => the subject alternative name
 #
 # Exceptions:
 #
@@ -1103,6 +1135,9 @@ sub listCertificates
 	$element{'path'} = CERTSDIR . $line[SERIAL_IDX] . ".pem";
       }
 
+      # Setting the subject alternative names
+      $element{'subjAltNames'} = $self->_obtain( $element{'path'}, 'subjAltNames' );
+
       push (@out, \%element);
 
     }
@@ -1149,6 +1184,10 @@ sub listCertificates
 #       - reason     - reason to revoke if state is revoked
 #       - path       - certificate path
 #       - serialNumber - serial number within CA
+#       - subjAltNames - array ref the subject alternative names containing hash ref
+#                         with the following valid types:
+#                        type  => 'DNS', 'IP' and 'email'
+#                        value => the subject alternative name
 #
 # Exceptions:
 #
@@ -1266,6 +1305,36 @@ sub getKeys {
 
 }
 
+# Method: getP12KeyStore
+#
+#       Given the common name, it returns the PKCS12 keystore file path.
+#
+#       The PKCS12 is a common format used by web browser to store
+#       certificates and private keys.
+#
+# Parameters:
+#
+#       commonName - String the common name
+#
+# Returns:
+#
+#       String - the PKCS 12 key store path
+#
+sub getP12KeyStore
+{
+    my ($self, $commonName) = @_;
+
+    my $target = P12DIR . "${commonName}.p12";
+    if (-f $target) {
+        return $target;
+    } else {
+        my $privKey = $self->getKeys($commonName)->{privateKey};
+        my $certMetadata = $self->getCertificateMetadata(cn => $commonName);
+        return $self->_generateP12Store($privKey, $certMetadata);
+    }
+
+}
+
 # Method: removePrivateKey
 #
 #       Remove the private key from an user
@@ -1334,6 +1403,8 @@ sub removePrivateKey
 #       the certFile is passed *(Optional)*
 #
 #       force - this forces the revokation pass to do the renewal *(Optional)*
+#       subjAltNames - Array ref containing hash ref with key type of certificate
+#                      and value the value for the subject alt name *(Optional)*
 #
 # Returns:
 #
@@ -1416,22 +1487,22 @@ sub renewCertificate
     }
     if (defined($args{stateName})
 	and $args{stateName} ne $userDN->attribute('stateName')) {
-      $dnFieldHasChanged = "1" ;
+      $dnFieldHasChanged = "1";
       $userDN->attribute('stateName', $args{stateName});
     }
     if (defined($args{localityName})
 	and $args{localityName} ne $userDN->attribute('localityName')) {
-      $dnFieldHasChanged = "1" ;
+      $dnFieldHasChanged = "1";
       $userDN->attribute('localityName', $args{localityName});
     }
     if (defined($args{orgName})
 	and $args{orgName} ne $userDN->attribute('orgName')) {
-      $dnFieldHasChanged = "1" ;
+      $dnFieldHasChanged = "1";
       $userDN->attribute('orgName', $args{orgName});
     }
     if (defined($args{orgNameUnit})
 	and $args{orgNameUnit} ne $userDN->attribute('orgNameUnit')) {
-      $dnFieldHasChanged = "1" ;
+      $dnFieldHasChanged = "1";
       $userDN->attribute('orgNameUnit', $args{orgNameUnit});
     }
 
@@ -1462,6 +1533,14 @@ sub renewCertificate
       $newCertFile = $userCertFile;
     }
 
+    my $subjAltNames;
+    if (defined($args{subjAltNames})) {
+        $subjAltNames = $args{subjAltNames};
+        $dnFieldHasChanged = "1";
+    } else {
+        $subjAltNames = $self->_obtain($userCertFile, 'subjAltNames');
+    }
+
     if (-f $userReq) {
       # If the request exists, we can renew the certificate without
       # having the private key
@@ -1473,8 +1552,8 @@ sub renewCertificate
 	# For OpenSSL 0.9.7, we need to create the request
 	my $privKeyFile = PRIVDIR . $userDN->attribute('commonName') . ".pem";
 	$privKeyFile = $args{privateKeyFile} if ($args{privateKeyFile});
-	throw EBox::Exceptions::External(__("The private key passpharse needed to create a new request. No renewal was made. Issue a new certificate with new keys"))
-	  if ( not defined($args{keyPassword}) );
+	#throw EBox::Exceptions::External(__("The private key passpharse needed to create a new request. No renewal was made. Issue a new certificate with new keys"))
+	#  if ( not defined($args{keyPassword}) );
 
 	my $output = $self->_createRequest(reqFile     => $userReq,
 					   genKey      => 0,
@@ -1499,6 +1578,7 @@ sub renewCertificate
 					createSerial => 0,
 # Not in OpenSSL 0.9.7			newSubject   => $newSubject,
 					endDate      => $userExpDay,
+                                        subjAltNames => $subjAltNames,
 				      );
 
       if (defined($output) ) {
@@ -1523,7 +1603,9 @@ sub renewCertificate
 			       privateKeyFile => $privKeyFile,
 			       requestFile    => $userReq,
 			       certFile       => $newCertFile,
-			       endDate        => $args{endDate});
+			       endDate        => $userExpDay,
+                               subjAltNames   => $subjAltNames,
+                              );
     }
 
     if (not defined($newCertFile) ) {
@@ -1699,7 +1781,7 @@ sub getCurrentCRL
 
 # Method: menu
 #
-#       Add CA module to eBox menu
+#       Add CA module to Zentyal menu
 #
 # Parameters:
 #
@@ -1732,24 +1814,35 @@ sub menu
 #       dir - Directory where the modules backup files are dumped
 #
 sub dumpConfig
-  {
-
+{
     my ($self, $dir) = @_;
 
     # Call super
     $self->SUPER::dumpConfig($dir);
 
     if ( -d CATOPDIR ) {
-      # Storing all OpenSSL directory tree where Certs/keys and DB are stored
-      dircopy(CATOPDIR, $dir . "/CA");
+        # Storing all OpenSSL directory tree where Certs/keys and DB are stored
+        dircopy(CATOPDIR, $dir . "/CA");
     }
 
     if ( -f SSLCONFFILE ) {
-      # Storing OpenSSL config file
-      fcopy(SSLCONFFILE, $dir . "/openssl.cnf");
+        # Storing OpenSSL config file
+        fcopy(SSLCONFFILE, $dir . "/openssl.cnf");
     }
 
-  }
+    # Dump services certificates
+    my @srvCerts = @{EBox::CA::Certificates->srvsCerts()};
+    my @paths;
+    foreach my $certificate (@srvCerts) {
+        if (-f $certificate->{'path'}) {
+            push(@paths, $certificate->{'path'});
+        }
+    }
+    if (@paths) {
+        my $pathsStr = join(' ', @paths);
+        EBox::Sudo::root("tar cf $dir/srvCerts.tar $pathsStr");
+    }
+}
 
 # Method:  restoreConfig
 #
@@ -1760,7 +1853,7 @@ sub dumpConfig
 #  dir - Directory where are located the backup files
 #
 sub restoreConfig
-  {
+{
 
     my ($self, $dir) = @_;
 
@@ -1771,13 +1864,29 @@ sub restoreConfig
     # Restoring all OpenSSL directory tree where Certs/keys and DB are stored
     my $dataDir = $dir . '/CA';
     if (-d $dataDir) {
-      dircopy($dataDir, CATOPDIR);
+        dircopy($dataDir, CATOPDIR);
     }
 
     # Restoring OpenSSL config file
     fcopy($dir . "/openssl.cnf", SSLCONFFILE);
 
-  }
+    # Restore services certificates
+    if (-f "$dir/srvCerts.tar") {
+        my @commands;
+        push(@commands, "tar xfp $dir/srvCerts.tar -C /");
+
+        my @srvCerts = @{EBox::CA::Certificates->srvsCerts()};
+        foreach my $certificate  (@srvCerts) {
+            my $certPath = $certificate->{'path'};
+            if (-f $certPath) {
+                my $certUser = $certificate->{'user'};
+                my $certGroup = $certificate->{'group'};
+                push(@commands, "chown $certUser:$certGroup $certPath");
+            }
+        }
+        EBox::Sudo::root(@commands);
+    }
+}
 
 # Method: showModuleStatus
 #
@@ -1820,6 +1929,49 @@ sub addModuleStatus
 
 }
 
+# Method: report
+#
+# Returns:
+#    ref hash with the following fields:
+#    CAState: state of the CA certificate. It will be ono of this states:
+#       - R - Revoked
+#       - E - Expired
+#       - V - Valid
+#       - ! - Inexistent
+#
+#    nValidCertificates: number of valid active certificates
+#    nRevokedCertificates: number of revoked certificates
+#    nExpiredCertificates: number of expired certifcates
+#
+# Overrides:
+#
+#   <EBox::Module::Base::report>
+#
+sub report
+{
+    my ($self) = @_;
+    my $currentCA = $self->currentCACertificateState();
+
+    my ($nValid, $nRevoked, $nExpired) = (0, 0, 0);
+    if ($currentCA ne '!') {
+        $nValid = @{ $self->listCertificates(state => 'V', excludeCA => 1) };
+        $nRevoked = @{ $self->listCertificates(state => 'R', excludeCA => 1) };
+        $nExpired = @{ $self->listCertificates(state => 'E', excludeCA => 1) };
+
+    }
+
+    return {
+            CAState => $currentCA,
+
+            nValidCertifcates     => $nValid,
+            nRevokedCertificates  => $nRevoked,
+            nExpiredCertificates  => $nExpired,
+           };
+}
+
+
+# Group: Protected methods
+
 # Method: _supportActions
 #
 #   Overrides <EBox::Module::ServiceBase>
@@ -1832,8 +1984,6 @@ sub _supportActions
     return undef;
 }
 
-# Group: Private methods
-
 # Method: _setConf # # Overrides: # # <EBox::Module::Service::_setConf> #
 sub _setConf {
 
@@ -1842,6 +1992,8 @@ sub _setConf {
     EBox::CA::Certificates->genCerts();
 }
 
+# Group: Private methods
+
 # Obtain the public key given the private key
 #
 # Return public key path if it is correct or undef if password is
@@ -1874,6 +2026,33 @@ sub _getPubKey # (privKeyFile, pubKeyFile, password?)
 
   }
 
+# Generate the pkcs12 key store with password if required
+sub _generateP12Store
+{
+    my ($self, $privKeyFile, $certMetadata, $password) = @_;
+
+    my $certFile = $certMetadata->{path};
+    my $cn       = $certMetadata->{dn}->attribute('commonName');
+    my $target   = P12DIR . "${cn}.p12";
+
+    my $cmd = 'pkcs12 -export ';
+    $cmd .= '-in '    . $certFile . ' ';
+    $cmd .= '-inkey ' . $privKeyFile . ' ';
+    $cmd .= '-name '  . $cn . ' ';
+    $cmd .= '-out '   . $target . ' ';
+    $password = '' unless (defined($password));
+    $cmd .= "-passout pass:$password ";
+
+    $ENV{'PASS'} = $password
+      if (defined($password));
+    my ($retVal) = $self->_executeCommand(COMMAND => $cmd);
+    delete( $ENV{'PASS'} );
+
+    return undef if ($retVal eq "ERROR");
+
+    return $target;
+}
+
 # Given a serial number, it returns the file path
 sub _certFile {
 
@@ -1968,7 +2147,7 @@ sub _createRequest # (reqFile, genKey, privKey, keyPassword, dn, needPass?)
 # return the output error if any error occurr, nothing otherwise
 # ? Optional Parameter
 sub _signRequest # (userReqFile, days, userCertFile?, policy?, selfsigned?,
-                 # newSubject?, endDate?)
+                 # newSubject?, endDate?, subjAltNames?)
   {
 
     my ($self, %args) = @_;
@@ -2014,7 +2193,13 @@ sub _signRequest # (userReqFile, days, userCertFile?, policy?, selfsigned?,
     $cmd .= "-outdir " . CERTSDIR . " ";
     $cmd .= "-out \'$args{userCertFile}\' " if defined($args{userCertFile});
     $cmd .= '-extensions v3_ca ' if ( EXTENSIONS_V3 and $args{selfsigned});
-    $cmd .= '-extensions usr_cert ' if ( EXTENSIONS_V3 and not $args{selfsigned});
+    my $tmpFileName;
+    if ( defined($args{subjAltNames}) ) {
+        $tmpFileName = $self->_generateExtFile($args{subjAltNames});
+        $cmd .= "-extfile '$tmpFileName' ";
+    } else {
+        $cmd .= '-extensions usr_cert ' if ( EXTENSIONS_V3 and not $args{selfsigned});
+    }
     # Only available in OpenSSL 0.9.8
     $cmd .= "-selfsign " if ($args{selfsigned});
     $cmd .= "-policy $policy ";
@@ -2032,6 +2217,9 @@ sub _signRequest # (userReqFile, days, userCertFile?, policy?, selfsigned?,
     my ($retVal, $output) = $self->_executeCommand(COMMAND => $cmd);
     delete ( $ENV{'PASS'} );
 
+    # Remove ext file if exists
+    # unlink( $tmpFileName ) if ( -e $tmpFileName );
+
     return $self->_filterErrorFromOpenSSL($output) if ($retVal eq "ERROR");
     return undef;
 
@@ -2101,6 +2289,8 @@ sub _commonArgs # (cmd, args)
 # attribute => 'serial' return => String containing the serial number
 # attribute =>'endDate' return => Date::Calc::Object with the date
 # attribute => 'days' return => number of days from today to expire
+# attribute => 'subjAltNames' return => Array ref containing hash ref with key type of certificate
+#                                       and value the value for the subject alt name
 # undef => if certification file does NOT exist
 
 sub _obtain # (certFile, attribute)
@@ -2120,6 +2310,8 @@ sub _obtain # (certFile, attribute)
     } elsif ($attribute eq 'endDate'
 	    or $attribute eq 'days') {
       $arg = "-enddate";
+    } elsif ($attribute eq 'subjAltNames' ) {
+        $arg = '-text -certopt -no_header,no_version,no_serial,no_signame,no_validity,no_subject';
     }
     my $cmd = "x509 " . $arg . " -in \'$certFile\' -noout";
 
@@ -2154,6 +2346,21 @@ sub _obtain # (certFile, attribute)
       my $certDate = $self->_parseDate($output);
       my $diffDate = $certDate - Date::Calc->Today();
       return $diffDate->day();
+    } elsif ($attribute eq 'subjAltNames') {
+        my ($altNameStr) = $output =~ m/X509v3 Subject Alternative Name:\s+(.*)\s+X509v3 Au/s;
+        return [] if not defined($altNameStr);
+        $altNameStr =~ s/\s+//g;
+        my @retValue = ();
+        my @subjAltNames = split(/,/, $altNameStr);
+        foreach my $subAlt (@subjAltNames) {
+            my ($type, $value) = split(/:/, $subAlt);
+            if ( $type eq 'IPAddress' ) {
+                push(@retValue, { type => 'IP', value => $value });
+            } else {
+                push(@retValue, { type => $type, value => $value });
+            }
+        }
+        return \@retValue;
     }
 
   }
@@ -2322,7 +2529,7 @@ sub _filterErrorFromOpenSSL # (input)
 
   }
 
-# Check none of other eBox modules uses the certificate
+# Check none of other Zentyal modules uses the certificate
 # Return true if there's any module using it
 sub _certsInUse # (cn?, isCACert?)
   {
@@ -2437,44 +2644,47 @@ sub _isLaterThanCA # (date)
 
 }
 
-
-# Method: report
-#
-# Returns:
-#    ref hash with the following fields:
-#    CAState: state of the CA certificate. It will be ono of this states:
-#       - R - Revoked
-#       - E - Expired
-#       - V - Valid
-#       - ! - Inexistent
-#
-#    nValidCertificates: number of valid active certificates
-#    nRevokedCertificates: number of revoked certificates
-#    nExpiredCertificates: number of expired certifcates
-#
-
-# Overrides: 
-#   <EBox::Module::Base::report>
-sub report
+# Check the given subject alternative names are correct
+sub _checkSubjAltNames
 {
-    my ($self) = @_;
-    my $currentCA = $self->currentCACertificateState();
-
-    my ($nValid, $nRevoked, $nExpired) = (0, 0, 0);
-    if ($currentCA ne '!') {
-        $nValid = @{ $self->listCertificates(state => 'V', excludeCA => 1) };
-        $nRevoked = @{ $self->listCertificates(state => 'R', excludeCA => 1) };
-        $nExpired = @{ $self->listCertificates(state => 'E', excludeCA => 1) };
+    my ($self, $subjAltNames) = @_;
 
+    foreach my $subjAlt (@{$subjAltNames}) {
+        if ( $subjAlt->{type} eq 'DNS' ) {
+            EBox::Validate::checkDomainName($subjAlt->{value}, 'DNS value');
+        } elsif ( $subjAlt->{type} eq 'IP' ) {
+            EBox::Validate::checkIP($subjAlt->{value}, 'IP value');
+        } elsif ( $subjAlt->{type} eq 'email' ) {
+            # copy is an special value to get the email value from CN subject
+            if ( $subjAlt->{value} ne 'copy' ) {
+                EBox::Validate::checkEmailAddress($subjAlt->{value}, 'email value');
+            }
+        } else {
+            throw EBox::Exceptions::InvalidData(
+                data   => 'type',
+                value  => $subjAlt->{type},
+                advice => __x('Only {dns}, {ip} and {email} are valid subject alternative '
+                                . 'name types', dns => 'DNS', ip => 'IP', email => 'email'),
+               );
+        }
     }
 
-    return {
-            CAState => $currentCA,
+}
+
+# Generate the v3 extension conf file to write the subj alt names
+# the results is the temporary file path to write onto
+sub _generateExtFile
+{
+    my ($self, $subjAltNames) = @_;
+
+    my $tmpFile = new File::Temp(DIR => EBox::Config::tmp());
+    $tmpFile->unlink_on_destroy(0);
+    EBox::Module::Base::writeConfFileNoCheck($tmpFile->filename(),
+                                             'ca/v3_ext.mas',
+                                             [ 'subjAltNames' => $subjAltNames ],
+                                            );
+    return $tmpFile->filename();
 
-            nValidCertifcates     => $nValid,
-            nRevokedCertificates  => $nRevoked,
-            nExpiredCertificates  => $nExpired,
-           };
 }
 
 ## OpenSSL execution environment provided by OpenCA::OpenSSL
