@@ -1,4 +1,4 @@
-# Copyright (C) 2009 eBox Technologies S.L.
+# Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -215,15 +215,20 @@ sub _genCert
     print $tempfile_fh @privkeydata;
     close($tempfile_fh);
 
+    my @commands;
+
     my $user = $srvcert->{'user'};
     my $group = $srvcert->{'group'};
-    EBox::Sudo::root("/bin/chown $user:$group $tempfile");
+    push (@commands, "/bin/chown $user:$group $tempfile");
 
     my $mode = $srvcert->{'mode'};
-    EBox::Sudo::root("/bin/chmod $mode $tempfile");
+    push (@commands, "/bin/chmod $mode $tempfile");
 
     my $path = $srvcert->{'path'};
-    EBox::Sudo::root("mv -f $tempfile $path");
+    push (@commands, "mkdir -p `dirname $path`");
+    push (@commands, "mv -f $tempfile $path");
+
+    EBox::Sudo::root(@commands);
 }
 
 # Method: _modsService
