@@ -1,4 +1,3 @@
-# Copyright (C) 2007 Warp Networks S.L.
 # Copyright (C) 2009-2010 eBox Technologies S.L.
 #
 # This program is free software; you can redistribute it and/or modify
@@ -147,6 +146,36 @@ sub disableService
     }
 }
 
+# Method: setServiceRO
+#
+#       Set service as read-only.
+#
+sub setServiceRO
+{
+    my ($self, $service, $ro) = @_;
+
+    my $row = $self->find(service => $service);
+    if ($row) {
+        $row->setReadOnly($ro);
+        $row->store();
+    }
+}
+
+# Method: updateCN
+#
+#       Updates the CN in the certificate for the given service.
+#
+sub updateCN
+{
+    my ($self, $service, $cn) = @_;
+
+    my $row = $self->find(service => $service);
+    if ($row) {
+        $row->elementByName('cn')->setValue($cn);
+        $row->store();
+    }
+}
+
 # Method: certUsedByService
 #
 #       Returns if a given certificate Common Name is used by any
@@ -256,7 +285,7 @@ sub _table
         sortedBy           => 'module',
         modelDomain        => 'CA',
         help               => __('Here you may set certificates from this CA for those '
-                                 . 'secure services managed by eBox'),
+                                 . 'secure services managed by Zentyal'),
     };
 
     return $dataTable;
